const fs = require('fs');
const { executeSyncCmd } = require('./build-util');
const os = require('os');


/**
 *  To check the npm package installation successs or not
 * @param {*} path  File path where installation success message was written
 * @param {*} msg   Success messsage to confirm that package was installed
 * @returns boolean true/false
 */
const isNPMPackageExist = (path, msg) => {
    if (fs.existsSync(path)) {
        const successMsg = fs.readFileSync(path, { encoding: 'utf8', flag: 'r' });
        if (successMsg == msg) {
            return true;
        }

    } else {
        return false;
    }
}


/**
 *  To delete the folder and files recursively
 * @param {*} path Deletion folder path
 */
const removeDir = (path, msg) => {
    if (fs.existsSync(path)) {
        const files = fs.readdirSync(path)

        if (files.length > 0) {
            files.forEach(function (filename) {
                if (fs.statSync(path + "/" + filename).isDirectory()) {
                    removeDir(path + "/" + filename)
                } else {
                    fs.unlinkSync(path + "/" + filename)
                }
            })
        }
        fs.rmdirSync(path)
    } else {
        console.log(msg + " Directory deletion path not found.")
    }
}

/**
 *  Check  node modules package were installed or not
 *  If not delete existing packages
 *  Create dir for  packages with the version name
 *  Copy the generated angular app package.json to PATH_NPM_PACKAGE folder
 *  Run npm install
 *  If something goes wrong delete the broken npm packages
 *  Write  success file to be make sure it was installed successfully.
 */
const downloadNPMPackage = (packageInfo) => {
    const USER_INFO = os.userInfo();
    const PATH_NPM_PACKAGE = (packageInfo.downloadBaseDir || USER_INFO.homedir) + '/.wm/node_modules/' + packageInfo.name + '/' + packageInfo.version;
    const PATH_NPM_PACKAGE_SUCCESS = PATH_NPM_PACKAGE + '/.SUCCESS';

    // To check global app runtime node modules.
    if (!isNPMPackageExist(PATH_NPM_PACKAGE_SUCCESS, packageInfo.successMsg)) {

        removeDir(PATH_NPM_PACKAGE, packageInfo.infoMsg);

        fs.mkdirSync(PATH_NPM_PACKAGE, { recursive: true });
        let npmInstallCMD = 'npm install ';
        if (packageInfo.packageJsonFile && fs.existsSync(packageInfo.packageJsonFile)) {
            fs.copyFileSync(packageInfo.packageJsonFile, PATH_NPM_PACKAGE + '/package.json');
        } else {
            npmInstallCMD = 'npm init -y &&  ' + npmInstallCMD + packageInfo.scope + '/' + packageInfo.name + '@' + packageInfo.version;
        }

        executeSyncCmd('cd ' + PATH_NPM_PACKAGE + ' && ' + npmInstallCMD, () => {
            removeDir(PATH_NPM_PACKAGE, packageInfo.infoMsg);
            console.log(packageInfo.infoMsg + ' Something wrong with npm installation, deleted broken npm packages from the path ' + PATH_NPM_PACKAGE);
        },  packageInfo.infoMsg);

        fs.writeFileSync(PATH_NPM_PACKAGE_SUCCESS, packageInfo.successMsg);
    } else {
        console.log(packageInfo.infoMsg + ' Node packages already installed!');
    }

    return PATH_NPM_PACKAGE;

}


module.exports = {
    downloadNPMPackage
}